package business;

//import Decorator;

//package business;
import network.ConfigurationServerInfo;
import access.Database;

import java.util.Hashtable;
import java.util.Vector;
import java.rmi.Naming;
import java.rmi.RemoteException;

import query.Parse;

public class AuralizerManager
{
    private ConfigurationServerInfo objCSI = null;
    private LSLCreator objLSLCreator = null;
    private FileManager objFM = null;
    private Hashtable htLSLTokens = null;
    private EventSoundMappingGenerator objESMG= null;
    private rmi.AuralizerInterface objAI;



    public AuralizerManager()
    {
        Database objDB = new Database();
        objDB.createFolders();
        objCSI = new ConfigurationServerInfo();
        objLSLCreator = new LSLCreator();
        objFM = new FileManager();

        htLSLTokens = new Hashtable();
    }

    private boolean connectConfServer()
    {
        try
        {
            System.out.println(objCSI.getIPAddress() + ":" + objCSI.getPort());
            objAI = (rmi.AuralizerInterface)javax.rmi.PortableRemoteObject.narrow
                    (Naming.lookup("rmi://"+ objCSI.getIPAddress() + ":" + objCSI.getPort() + "/AURALISER_SERVER")
                     ,rmi.AuralizerInterface.class);

            if(objAI ==null)
            {
                System.out.println("AURALIZER NOT CONNECTED");
                return false;

            }

            return true;
        }
        catch(Exception e)
        {
            //e.printStackTrace();
            System.out.println("Exception at AURALISER Interface Communicator");
            return false;
        }
    }


    public void generateEventSoundMappings(String sProgramName,Vector vLSLTokens)
    {
        objESMG = new EventSoundMappingGenerator();
        objESMG.setProgramName(sProgramName);
        objESMG.generateEventSoundMappings(vLSLTokens);
    }

    public Hashtable getEventSoundMappings(String sProgramName)
    {
        return  objESMG.getEventSoundMappings(sProgramName);
    }

    public boolean registerAuralizedProgram(String sProgramName, Hashtable htEventSoundMapping)
    {
        connectConfServer();
        try{
            return objAI.registerAuralizedProgram(sProgramName, htEventSoundMapping);
        }
        catch(RemoteException re)
        {
            System.out.println("remote exception REGISTER AURALISER");
            return false;
        }
    }

    public boolean unRegisterAuralizedProgram(String sProgramName)
    {
        try{
            return objAI.unRegisterAuralizedProgram(sProgramName);
        }
        catch(RemoteException re)
        {
            System.out.println("remote exception UNREGISTER AURALISER");
            return false;
        }
    }

    public void setNetworkInfo(String sIPAddress, String sPort)
    {
        objCSI.setNetworkInfo(sIPAddress, sPort);
    }

    public String[] getNetworkInfo()
    {
        return objCSI.getNetworkInfo();
    }

    public String[] getEvents()
    {
        return objLSLCreator.getEvents();
    }

    public String[] getScopes()
    {
        return objLSLCreator.getScopes();
    }

    public Vector getInstrumentNames()
    {
        return objLSLCreator.getInstrumentNames();
    }

    public String[] getLSLBodyTemplate(String sJavaFileName)
    {
        return objLSLCreator.getLSLBodyTemplate(sJavaFileName);
    }

    public String createLSLStatement(LSLTokens objTokens)
    {
        return objLSLCreator.createLSLStatement(objTokens);
    }

    public boolean decorate(String sJavaFileName, String sJavaFilePath, Vector vLSLTokens)
    {
        Decorator objDecorator = new Decorator();
        Parse objParse=  objDecorator.decorate(sJavaFileName, sJavaFilePath, vLSLTokens);

        objFM.createInstrumentedPackage(sJavaFileName, objParse);

        return true;
    }



    public boolean storeLSLFile(String[] sLSL, String sJavaFileName)
    {
        return objFM.storeLSLFile(sLSL, sJavaFileName);
    }

    public LSLTokens createLSLToken(String sPackage, String sClass, String sMethod, Vector vMethodScopeParameters,
                                    String sKeyword, String sIdentifier, boolean bActivity, String sInstrument,
                                    Vector vParameters, int iNote)
    {
        LSLTokens objLSLTokens = new LSLTokens(sPackage, sClass, sMethod, vMethodScopeParameters,sKeyword, sIdentifier,
                                               bActivity, sInstrument, vParameters, iNote);
        return objLSLTokens;
    }

    public LSLTokens createLSLToken(String sPackage, String sClass, String sMethod,
                                    Vector vMethodScopeParameters, String sKeyword, String sIdentifier,Vector vValues,
                                    boolean bActivity, Vector vInstrument, Vector vParameters, Vector vNote,
                                    int iDataType, String sPatchMethod)
    {
        LSLTokens objLSLTokens = new LSLTokens(sPackage, sClass, sMethod, vMethodScopeParameters,sKeyword, vValues,
                                               bActivity, vInstrument, vParameters, vNote, iDataType, sPatchMethod, sIdentifier);
        return objLSLTokens;
    }

    public boolean storeLSLTokens(String sJavaFileName,Vector vLSLTokens)
    {
        Database objDB = new Database();

        try
        {
            htLSLTokens.put(sJavaFileName, vLSLTokens);
            objDB.writeObject(sJavaFileName,vLSLTokens);
        }
        catch(Exception e)
        {
            System.out.println("NULL LSL TOKENS");
            return false;
        }

        return true;
    }

    public Vector retrieveTokens(String sJavaFileName)
    {
        Database objDB = new Database();

        try
        {
            return (Vector) objDB.readObject(sJavaFileName);
        }
        catch(Exception e)
        {
            System.out.println("NULL LSL TOKENS");
            return null;
        }
    }

    public LSLTokens getLSLTokens(String sJavaFileName)
    {
        try
        {
            return (LSLTokens) htLSLTokens.get(sJavaFileName);
        }
        catch(Exception e)
        {
            System.out.println("LSL TOKEN NOT FOUND");
            return null;
        }
    }

}